(define (script-fu-antialias img layer weight)
  (plug-in-antialias 1 img layer weight))

(script-fu-register "script-fu-antialias"
		    "<Image>/Filters/Blur/Antialias..."
		    "UI for Antialias."
		    "Kevin Turner"
		    "Kevin Turner"
		    "1998"
		    "RGB*, GRAY*"
		    SF-IMAGE "Image" 0
		    SF-DRAWABLE "Layer" 0
		    SF-VALUE "Weight" "0.33333333")

